@@ -16,6 +16,7 @@
 
 #include "statisticsmainwindow.h"
 
+#include <QHeaderView>
 #include <QtCore/qsignalmapper.h>
 
 #include <KLocalizedString>
@@ -37,10 +38,11 @@
 #include "conjugationoptions.h"
 
 
-StatisticsMainWindow::StatisticsMainWindow(KEduVocDocument* doc, ParleyMainWindow* parent)
+StatisticsMainWindow::StatisticsMainWindow(KEduVocDocument *doc, ParleyMainWindow *parent)
     : KXmlGuiWindow(parent)
     , m_mainWindow(parent)
     , m_doc(doc)
+    , m_ui(new Ui::StatisticsMainWindow)
     , m_conjugationOptions(0)
 {
     // KXmlGui
@@ -49,7 +51,6 @@ StatisticsMainWindow::StatisticsMainWindow(KEduVocDocument* doc, ParleyMainWindo
 
     QWidget *mainWidget = new QWidget(this);
     setCentralWidget(mainWidget);
-    m_ui = new Ui::StatisticsMainWindow;
     m_ui->setupUi(mainWidget);
     //m_ui->caption->setText(i18nc("caption for an overview of the confidence levels for a document"
     //                             "Statistics for \"%1\"", m_doc->title()));
@@ -88,7 +89,7 @@ void StatisticsMainWindow::setDocument(KEduVocDocument* doc)
     m_doc = doc;
     m_statisticsModel->setDocument(doc);
     m_ui->lessonStatistics->setModel(m_statisticsModel);
-    m_ui->lessonStatistics->expandAll();
+    m_ui->lessonStatistics->restoreExpandedStatus();
     m_ui->lessonStatistics->resizeColumnToContents(0);
 }
 
@@ -195,6 +196,7 @@ void StatisticsMainWindow::languagesChanged()
     emit languagesChanged(knownLanguage, learningLanguage);
 
     updateVisibleColumns();
+    updateModelSettings();
 }
 
 void StatisticsMainWindow::initPracticeMode()
@@ -204,27 +206,55 @@ void StatisticsMainWindow::initPracticeMode()
     m_ui->practiceDirection->insertItem(2, i18n("Mixed Directions"));
     //m_ui->practiceDirection->insertItem(3, i18n("Mixed Directions with Sound"));
 
-    if (Prefs::practiceDirection() < 0 || 3 < Prefs::practiceDirection())
+    int practiceDirection(Prefs::rememberPracticeDirection()
+        ? practiceDirectionForPracticeMode(Prefs::practiceMode()) : Prefs::practiceDirection());
+
+    if (practiceDirection < 0 || 3 < practiceDirection)
         Prefs::setPracticeDirection(Prefs::EnumPracticeDirection::MixedDirectionsWordsOnly);
 
-    m_ui->practiceDirection->setCurrentIndex(Prefs::practiceDirection());
+    m_ui->practiceDirection->setCurrentIndex(practiceDirection);
+    connect(m_ui->practiceDirection, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+            this, &StatisticsMainWindow::practiceDirectionChanged);
 
-    connect(m_ui->practiceDirection, SIGNAL(currentIndexChanged(int)),
-            this,                    SLOT(practiceDirectionChanged(int)));
+    m_ui->rememberPracticeDirection->setChecked(Prefs::rememberPracticeDirection());
+    connect(m_ui->rememberPracticeDirection, &QCheckBox::toggled,
+            this, &StatisticsMainWindow::rememberPracticeDirectionChanged);
 }
 
 void StatisticsMainWindow::practiceModeSelected(int mode)
 {
+    int previousPracticeMode = Prefs::practiceMode();
     Prefs::setPracticeMode(static_cast<Prefs::EnumPracticeMode::type>(mode));
     //qDebug() << "mode: " << mode << Prefs::practiceMode();
 
     showConjugationOptions(mode == Prefs::EnumPracticeMode::ConjugationPractice);
+
+    if (Prefs::rememberPracticeDirection()) {
+        setPracticeDirectionForPracticeMode(Prefs::practiceDirection(), previousPracticeMode);
+        m_ui->practiceDirection->setCurrentIndex(practiceDirectionForPracticeMode(mode));
+    }
+
+    updateModelSettings();
 }
 
 void StatisticsMainWindow::practiceDirectionChanged(int mode)
 {
     //qDebug() << "new practice direction:" << mode;
     Prefs::setPracticeDirection(static_cast<Prefs::EnumPracticeDirection::type>(mode));
+    if (Prefs::rememberPracticeDirection()) {
+        setPracticeDirectionForPracticeMode(mode, Prefs::practiceMode());
+    }
+    updateVisibleColumns();
+    updateModelSettings();
+}
+
+void StatisticsMainWindow::rememberPracticeDirectionChanged(bool checked)
+{
+//     qDebug() << "remember practice direction changed to: " << checked;
+    Prefs::setRememberPracticeDirection(checked);
+    if (checked) {
+        setPracticeDirectionForPracticeMode(Prefs::practiceDirection(), Prefs::practiceMode());
+    }
 }
 
 void StatisticsMainWindow::updateVisibleColumns()
@@ -249,6 +279,7 @@ void StatisticsMainWindow::updateVisibleColumns()
         }
 
         m_ui->lessonStatistics->setColumnHidden(i, isHidden);
+        m_ui->lessonStatistics->adjustColumnWidths();
     }
 }
 
@@ -263,8 +294,11 @@ void StatisticsMainWindow::showConjugationOptions(bool visible)
         QHBoxLayout* layout = new QHBoxLayout(m_ui->modeSpecificOptions);
         layout->setMargin(0);
         layout->addWidget(m_conjugationOptions);
-        connect(this, SIGNAL(languagesChanged(int, int)), m_conjugationOptions, SLOT(setLanguages(int, int)));
+        connect(this, QOverload<int, int>::of(&StatisticsMainWindow::languagesChanged),
+                m_conjugationOptions, &ConjugationOptions::setLanguages);
         m_conjugationOptions->setLanguages(Prefs::knownLanguage(), Prefs::learningLanguage());
+        connect(m_conjugationOptions, &ConjugationOptions::checkBoxChanged,
+                this, &StatisticsMainWindow::updateModelSettings);
     }
     m_conjugationOptions->setVisible(visible);
 }
@@ -274,3 +308,31 @@ void StatisticsMainWindow::configurePractice()
     ConfigurePracticeDialog dialog(m_doc, this, QStringLiteral("practice settings"),  Prefs::self());
     dialog.exec();
 }
+
+int StatisticsMainWindow::practiceDirectionForPracticeMode(int mode) const
+{
+    int direction = Prefs::practiceDirectionsByPracticeMode().value(mode, Prefs::practiceDirection());
+    if ((direction < 0) || (direction > Prefs::EnumPracticeDirection::MixedDirectionsWordsOnly)) {
+        direction = Prefs::EnumPracticeDirection::MixedDirectionsWordsOnly;
+    }
+    return direction;
+}
+
+void StatisticsMainWindow::setPracticeDirectionForPracticeMode(int direction, int mode)
+{
+    QList<int> directions = Prefs::practiceDirectionsByPracticeMode();
+    // Expand list if fields not used before
+    for (int i = directions.size() - 1; i < mode; ++i) {
+        directions.append(Prefs::practiceDirection());
+    }
+    directions[mode] = direction;
+    Prefs::setPracticeDirectionsByPracticeMode(directions);
+}
+
+void StatisticsMainWindow::updateModelSettings()
+{
+    m_ui->lessonStatistics->saveExpandedStatus();
+    m_statisticsModel->updateDocumentSettings();
+    m_ui->lessonStatistics->restoreExpandedStatus();
+}
+
